#  openMat for blender - A demo to show how to use the 
#  import and export library 
#  for the openMat database format
#
#  Copyright (C) 2012 Stefan Drechsler
# ##### BEGIN GPL LICENSE BLOCK #####
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2
#  of the License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software Foundation,
#  Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# ##### END GPL LICENSE BLOCK #####

import openMat.Material
import math

def getSigma(material):
    a125 = getOktaveSigma(material, 125.0)
    a250 = getOktaveSigma(material, 250.0)
    a500 = getOktaveSigma(material, 500.0)
    a1k = getOktaveSigma(material, 1000.0)
    a2k = getOktaveSigma(material, 2000.0)
    a4k = getOktaveSigma(material, 4000.0)
    if ((a125 == None) or (a250 == None) or 
        (a500 == None) or (a1k == None) or 
        (a2k == None) or (a4k == None)):
        return
    return (a125, a250, a500, a1k, a2k, a4k)

def getOktaveSigma(material, frequency):
    sigData = material.getScatData()
    if sigData == None:
        return
    return interpolate(sigData, frequency)    

def getAlpha(material):
    a125 = getOktaveAlpha(material, 125.0)
    a250 = getOktaveAlpha(material, 250.0)
    a500 = getOktaveAlpha(material, 500.0)
    a1k = getOktaveAlpha(material, 1000.0)
    a2k = getOktaveAlpha(material, 2000.0)
    a4k = getOktaveAlpha(material, 4000.0)
    if ((a125 == None) or (a250 == None) or 
        (a500 == None) or (a1k == None) or 
        (a2k == None) or (a4k == None)):
        return
    return (a125, a250, a500, a1k, a2k, a4k)

def getOktaveAlpha(material, frequency):
    absData = material.getAbsoData()
    if absData == None:
        return
    return interpolate(absData, frequency)    

def interpolate(data, frequency):
    distances = data.getDistances()
        
    nDist = len(distances)
        
    if nDist <= 0:
        return
        
    if nDist == 1:
        iSoFar = 0
    else:
        distSoFar = 0.0
        iSoFar = -1
        for i in range(nDist):
            d = float(distances[i].getD())
            if d == '':
                distSoFar = -1.0
                iSoFar = i
                break
            if d > distSoFar:
                distSoFar = d
                iSoFar = i
    distEle = distances[iSoFar]
        
    spektrum = []
    for freqEle in distEle.getFrequencies():
        f = float(freqEle.getFreq())
        alpha = interpolAngle(freqEle.getAngles())
        spektrum.append((f, alpha))
            
    w2 = math.sqrt(2.0)    
    frequencyMin = frequency / w2
    frequencyMax = frequency * w2  
        
    value = 0.0
    n = 0
    # In a first approach only a simple averaging scheme is used.
    # TODO this has to be refined.
    for item in spektrum:
        fItem = item[0]
        if frequencyMin <= fItem <= frequencyMax:
            value = value + item[1]
            n = n + 1
    # if there are no data in this octave band then the worst case is assumed
    if n == 0:
        alpha = 0.0
    else:
        alpha = value / n
    if alpha > 1.0:
        alpha = 1.0
    if alpha < 0.0:
        alpha = 0.0
    return alpha
    
        
def interpolAngle(angles):
    # For simplicity it is assumed that the angles are equally distributed so that
    # a mean value instead of a weighted average is used.
    # Of course this assumption is wrong.
    alpha = 0.0
    n = 0
    for angle in angles:
        tmp = float(angle.getValue1())
        alpha = alpha + tmp 
        n = n + 1
    return alpha/n