#  openMat for blender - A demo to show how to use the 
#  import and export library 
#  for the openMat database format
#
#  Copyright (C) 2012 Stefan Drechsler
# ##### BEGIN GPL LICENSE BLOCK #####
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2
#  of the License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software Foundation,
#  Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# ##### END GPL LICENSE BLOCK #####



import bpy
import bpy.props
import bpy.utils
import bpy.types

# <pep8-80 compliant>

bl_info = {
    "name": "roomacoustics",
    "author": "Stefan Drechsler",
    "version": (0, 8),
    "blender": (2, 6, 3),
    "location": "Properties -> Material",
    "description": "Add acoustical material properties, "
                   "absorption and scattering coefficients",
    "warning": "Still in development",
    "wiki_url": "http://openmat.info",
    "tracker_url": "",
    "support": 'TESTING',
    "category": "Physics"}

from acousticAddons.sabine import SabineOperator
from acousticAddons.sabine import AcousticObjectPanel
from acousticAddons.importOpenMat import IMPORT_OT_openMat

    

import mathutils
import math
    
    

bpy.types.Material.acoustic_alpha = bpy.props.FloatVectorProperty(
    size=6, default = (0.5, 0.5, 0.5, 0.5, 0.5, 0.5),
    min = 0.0, max = 1.0)
bpy.types.Material.acoustic_scatter = bpy.props.FloatVectorProperty(
    size=6, default = (0.5, 0.5, 0.5, 0.5, 0.5, 0.5),
    min = 0.0, max = 1.0)
bpy.types.Material.acoustic_description = bpy.props.StringProperty()


bpy.types.Scene.acoustic_air_damping = bpy.props.FloatVectorProperty(
    size=6, default = (0.00064, 0.00064, 0.00064, 0.00111, 0.00214, 0.00591), 
    min = 0.0)
    #Kuttruff: "Room Acoustics, Table 6.1, page 150
                                    
class AcousticMaterialPanel(bpy.types.Panel):
    bl_label = "Acoustics"
    bl_idname = "MATERIAL_PT_acoustics"
    bl_space_type = "PROPERTIES"
    bl_region_type = "WINDOW"

    bl_context = "material"

    def draw(self, context):
        layout = self.layout

        mat = context.material

        row = layout.row()
        row.label(text="Active object is: " + mat.name)

        split = layout.split()
        
        col = split.column()
        col.label(text="octave band")
        col.label(text="125Hz")
        col.label(text="250Hz")
        col.label(text="500Hz")
        col.label(text="1000Hz")
        col.label(text="2000Hz")
        col.label(text="4000Hz")

        col = split.column()
        col.prop(mat, "acoustic_alpha", text="absorption coefficient", slider=True)

        col = split.column()
        col.prop(mat, "acoustic_scatter", text="scatter coefficient", slider=True)
        
        row = layout.row()
        #row.label(text=mat.acoustic_description)
        row.prop(mat, "acoustic_description", text="description")
        # TODO textbox statt label


class AcousticParameterPanel(bpy.types.Panel):
    bl_label = 'Acoustics'
    bl_idname = 'SCENE_PT_acoustics'
    bl_space_type = 'PROPERTIES'
    bl_region_type = 'WINDOW'
    bl_context = 'scene'
    
    def draw(self, context):
        layout = self.layout
        scene = context.scene
        row = layout.row()
        row.label(text="Common Parameters:")
        row = layout.row()
        split = row.split()
        
        #col = split.column()
        #col.prop(scene, "audio_doppler_speed", text="sound speed c")

        col = split.column()
        col.prop(scene, "acoustic_air_damping", text="air damping m")

        row = layout.row()
        split = row.split()
        
def register():
    bpy.utils.register_class(AcousticMaterialPanel)
    bpy.utils.register_class(AcousticParameterPanel)
    
    bpy.utils.register_class(SabineOperator)
    bpy.utils.register_class(AcousticObjectPanel)
    
    bpy.utils.register_class(IMPORT_OT_openMat)
    
    


def unregister():
    bpy.utils.unregister_class(AcousticMaterialPanel)
    bpy.utils.unregister_class(AcousticParameterPanel)

    bpy.utils.unregister_class(SabineOperator)
    bpy.utils.unregister_class(AcousticObjectPanel)
    
    bpy.utils.unregister_class(IMPORT_OT_openMat)
    